@@ -7,7 +7,7 @@ use Module::Runtime qw(require_module);
 use Moo;
 use 5.10.1;
 
-our $VERSION = '0.0.16';
+our $VERSION = '0.0.17';
 
 BEGIN {
     if (my $trace_var = $ENV{PERL_ZOOKEEPER_TRACE}) {
@@ -106,6 +106,23 @@ ZooKeeper writes to a Unix pipe with an attached AnyEvent I/O watcher. This mean
 
 ZooKeeper uses Async::Interrupt callbacks. This means the perl interpreter will be safely interrupted(waits for the current op to finish) in order to execute the corresponding perl callback. See Async::Interrupt for more details on how callbacks are executed. Be aware that this does not interrupt system calls(such as select) and XS code. This means if your code is blocking on a select(such as during an AnyEvent recv), the interrupt callback will not execute until the call has finished.
 
+=item IOAsync
+
+ZooKeeper writes to a Unix pipe with an attached IO::Async::Handle.
+
+The IO::Async dispatcher requires an IO::Async::Loop, and needs to be constructed manually
+
+    my $loop = IO::Async::Loop->new;
+    my $disp = ZooKeeper::Dispatcher::IOAsync->new(loop => $loop);
+    my $zk = ZooKeeper->new(
+        hosts      => 'localhost:2181',
+        dispatcher => $disp,
+    );
+
+=item POE
+
+ZooKeeper writes to a Unix pipe with an attached POE::Session.
+
 =back
 
 =head1 ATTRIBUTES
@@ -202,38 +219,34 @@ Valid types include:
 
 =item Interrupt
 
+=item POE
+
 =back
 
+Instead of a string, a dispatcher object can be passed directly. This is necessary if the dispatcher has required attributes(as is the case for ZooKeeper::Dispatcher::IOAsync).
+
 =cut
 
 has dispatcher => (
     is      => 'ro',
+    isa     => sub { shift->isa('ZooKeeper::Dispatcher') },
+    coerce  => \&_to_dispatcher,
     default => sub { $ENV{PERL_ZOOKEEPER_DISPATCHER} || 'AnyEvent' },
-);
-
-has _dispatcher_obj => (
-    is       => 'ro',
-    init_arg => undef,
-    lazy     => 1,
-    builder  => '_build_dispatcher_obj',
-    handles  => [qw(
+    handles => [qw(
         create_watcher
         get_watchers
+        ignore_session_events
         remove_watcher
         wait
     )],
 );
+sub _to_dispatcher {
+    my ($disp) = @_;
+    return $disp unless $disp and not ref $disp;
 
-sub _build_dispatcher_obj {
-    my ($self) = @_;
-    my $type  = lc $self->dispatcher;
-    my $class = $type eq 'anyevent'  ? 'ZooKeeper::Dispatcher::AnyEvent'
-
-              : $type eq 'interrupt' ? 'ZooKeeper::Dispatcher::Interrupt'
-              : croak "Unrecognized dispatcher type: $type";
-
+    my $class = $disp =~ s/\^+// ? $disp : "ZooKeeper::Dispatcher::$disp";
     require_module($class);
-    return $class->new(ignore_session_events => $self->ignore_session_events);
+    return $class->new;
 }
 
 =head2 ignore_session_events
@@ -244,20 +257,17 @@ The default value is true, which will only trigger watchers once, for the watche
 
 =cut
 
-has ignore_session_events => (
-    is      => "ro",
-    default => 1,
-);
-
 sub BUILD {
     my ($self, $args) = @_;
-    my $default_watcher = $self->watcher ? $self->create_watcher('' => $self->watcher, type => 'default') : undef;
 
-    $self->_xs_init($self->hosts, $self->timeout, $default_watcher, $args->{client_id});
+    $self->ignore_session_events($args->{ignore_session_events})
+        if exists $args->{ignore_session_events};
+    my $watcher = $self->watcher && do {
+        $self->create_watcher('' => $self->watcher, type => 'default');
+    };
 
-    if (my $auth = $self->authentication) {
-        $self->add_auth(@$auth);
-    }
+    $self->_xs_init($self->hosts, $self->timeout, $watcher, $args->{client_id});
+    $self->add_auth(@{$self->authentication||[]}) if $self->authentication;
 }
 
 =head1 METHODS
