@@ -13,7 +13,7 @@ ZooKeeper::Dispatcher
 
 =head1 DESCRIPTION
 
-A parent class for event dispatchers to inherit from. Dispatchers directly handle callbacks for ZooKeeper C library, and manage the lifecycle of ZooKeeper::Watcher's.
+A parent class for event dispatchers to inherit from. Dispatchers directly handle callbacks for ZooKeeper the C library, and manage the lifecycle of ZooKeeper::Watcher's.
 
 =head1 ATTRIBUTES
 
@@ -64,7 +64,7 @@ Controls whether watchers should be triggered for session events.
 =cut
 
 has ignore_session_events => (
-    is      => "ro",
+    is      => "rw",
     default => 1,
 );
 
@@ -193,13 +193,14 @@ sub wait {
     my $cv = AnyEvent->condvar;
     my $w; $w = AnyEvent->timer(after => $time, cb => sub { $cv->send }) if $time;
 
+    my $dispatch_cb = $self->dispatch_cb;
     $self->dispatch_cb(sub {
-        my $event = $self->dispatch_event;
+        my $event = $dispatch_cb->();
         $cv->send($event);
     });
     my $event = $cv->recv;
 
-    $self->dispatch_cb(sub { $self->dispatch_event });
+    $self->dispatch_cb($dispatch_cb);
 
     weaken($self);
     return $event;
