@@ -25,15 +25,17 @@ has interrupt => (
 
 sub _build_interrupt {
     my ($self) = @_;
+    my $interrupt = Async::Interrupt->new(cb => sub { $self->dispatch_cb->() });
+
     weaken($self);
-    return Async::Interrupt->new(cb => sub { $self->dispatch_cb->() });
+    return $interrupt;
 }
 
 around wait => sub {
-    my ($orig, $self) = @_;
+    my ($orig, $self, @args) = @_;
     my $tick   = 0.1;
     my $ticker = AnyEvent->timer(after => $tick, interval => $tick, cb => sub {});
-    $self->$orig();
+    $self->$orig(@args);
 };
 
 sub BUILD {
